#################################################################################
## GPo 2022, AvsPmod macro 'Run analysis pass.py'
## Like menu 'Run analysis pass' but in a separate thread. 
## Therefore you can continue working with AvsPmod on another tab (script).
## But it is a bit slower than 'Run analysis pass' from the AvsPmod tools menu. 
#################################################################################

# run macro in new thread
import pyavs
import time
import tempfile

self = avsp.GetWindow()
script = self.currentScript
if self.readFrameProps:
	self.propWindow.Close()
if script.AVI:
    if not avsp.SafeCall(self.OnMenuScriptReleaseMemory, event=None, show=False):
        avsp.SafeCall(avsp.MsgBox, 'Cannot close the current script', 'Analysis pass error')
        return
      
filename = workdir = script.filename
if not filename:
    workdir = tempfile.gettempdir()
    
progress = avsp.ProgressBox(message='Starting analysis pass...', title='Analysis pass')
AVI = pyavs.AvsSimpleClipBase(avsp.GetText(), filename=filename, workdir=workdir)
if AVI is None:
    progress.Destroy()
    avsp.SafeCall(avsp.MsgBox,'Cannot create AvsSimpleClipBase', 'Analysis pass error')
    return
if AVI.error_message is not None:
    progress.Destroy()
    AVI = None
    avsp.SafeCall(avsp.MsgBox, AVI.error_message, 'Analysis pass error')
    return

frame_count = AVI.Framecount
previous_frame = -1
#progress = avsp.ProgressBox(message='Starting analysis pass...', title='Analysis pass')
initial_time = previous_time = time.time()
for frame in range(frame_count):
    AVI.clip.get_frame(frame)
    error = AVI.clip.get_error()
    if error:
        progress.Destroy()
        AVI = None
        avsp.SafeCall(avsp.MsgBox, u'\n\n'.join((_('Error requesting frame {number}').format(number=frame), error)), 'Error')
        return
    now = time.time()
    delta = now - previous_time
    elapsed_time = now - initial_time
    if delta >= 0.1: # then elapsed_time > 0, so no check is needed (if elapsed_time else 'INF')
        fps = (frame - previous_frame) / delta
        previous_time = now
        previous_frame = frame
        if not avsp.SafeCall(progress.Update, frame * 100/ frame_count, _('Average %#.4g fps\nFrame %s/%s (%#.4g fps)') %  
                              ((frame+1)/ elapsed_time, frame, frame_count, fps))[0]:
            progress.Destroy()
            AVI = None
            return
elapsed_time = time.time() - initial_time
AVI = None
avsp.SafeCall(progress.Update, 100, 'Finished (%s fps average)\n*** live and let live ***' % (
                '%#.4g' % (frame_count / elapsed_time) if elapsed_time else 'INF'))
progress.Destroy()  